// -*- C++ -*-
/*!
 * @file  Flip.cpp
 * @brief Flip image component
 * @date $Date$
 *
 * $Id$
 */


#include "Flip.h"
#include <iostream>
using namespace std;

// Module specification
// <rtc-template block="module_spec">
static const char* flip_spec[] =
  {
    "implementation_id", "Flip",
    "type_name",         "Flip",
    "description",       "Flip image component",
    "version",           "1.0.0",
    "vendor",            "AIST",
    "category",          "Category",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    "exec_cxt.periodic.rate", "1000.0",
    // Configuration variables
    "conf.default.flip_mode", "1",// Widget
    "conf.__widget__.flip_mode", "radio",
    // Constraints
    "conf.__constraints__.flip_mode", "(-1,0,1)",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
Flip::Flip(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_image_origIn("original_image", m_image_orig),
    m_image_flipOut("fliped_image", m_image_flip)
    // </rtc-template>
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  registerInPort("original_image", m_image_origIn);
  
  // Set OutPort buffer
  registerOutPort("fliped_image", m_image_flipOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

}

/*!
 * @brief destructor
 */
Flip::~Flip()
{
}



RTC::ReturnCode_t Flip::onInitialize()
{
  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("flip_mode", m_flip_mode, "1");
  
  // </rtc-template>
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t Flip::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Flip::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Flip::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t Flip::onActivated(RTC::UniqueId ec_id)
{
  // C[Wp̏
  m_image_buff = NULL;
  m_flip_image_buff = NULL;

  // OutPort̉ʃTCY̏
  m_image_flip.width = 0;
  m_image_flip.height = 0;

  return RTC::RTC_OK;
}


RTC::ReturnCode_t Flip::onDeactivated(RTC::UniqueId ec_id)
{
  if(m_image_buff != NULL)
  {
    // C[Wp̉
    cvReleaseImage(&m_image_buff);
    cvReleaseImage(&m_flip_image_buff);
  }

  return RTC::RTC_OK;
}


RTC::ReturnCode_t Flip::onExecute(RTC::UniqueId ec_id)
{
	// Vf[^̃`FbN
  if (m_image_origIn.isNew()) {
    // InPortf[^̓ǂݍ
    m_image_origIn.read();

	// InPortOutPort̉ʃTCYуC[Wp̊m
	if( m_image_orig.width != m_image_flip.width || m_image_orig.height != m_image_flip.height)
	{
		m_image_flip.width = m_image_orig.width;
		m_image_flip.height = m_image_orig.height;

		// InPort̃C[WTCYύXꂽꍇ
		if(m_image_buff != NULL)
		{
			cvReleaseImage(&m_image_buff);
			cvReleaseImage(&m_flip_image_buff);
		}

		// C[Wp̊m
		m_image_buff = cvCreateImage(cvSize(m_image_orig.width, m_image_orig.height), IPL_DEPTH_8U, 3);
		m_flip_image_buff = cvCreateImage(cvSize(m_image_orig.width, m_image_orig.height), IPL_DEPTH_8U, 3);
	}

    // InPort̉摜f[^IplImageimageDataɃRs[
    memcpy(m_image_buff->imageData,(void *)&(m_image_orig.pixels[0]),m_image_orig.pixels.length());

    // InPort̉摜f[^𔽓]B m_flip_mode 0: X, 1: Y, -1: ̎
    cvFlip(m_image_buff, m_flip_image_buff, m_flip_mode);

    // 摜f[^̃TCY擾
    int len = m_flip_image_buff->nChannels * m_flip_image_buff->width * m_flip_image_buff->height;
    m_image_flip.pixels.length(len);

    // ]摜f[^OutPortɃRs[
    memcpy((void *)&(m_image_flip.pixels[0]),m_flip_image_buff->imageData,len);

    // ]摜f[^OutPorto͂B
    m_image_flipOut.write();
  }

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t Flip::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Flip::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Flip::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Flip::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t Flip::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void FlipInit(RTC::Manager* manager)
  {
    coil::Properties profile(flip_spec);
    manager->registerFactory(profile,
                             RTC::Create<Flip>,
                             RTC::Delete<Flip>);
  }
  
};


